package at.gv.bmi.szr.client;

import iaik.security.rsa.RSAPrivateKey;
import iaik.utils.Base64InputStream;
import iaik.utils.Base64OutputStream;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.util.Iterator;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import at.gv.bmi.szr.PersonInfoType;
import at.gv.bmi.szr.TravelDocumentType;
import at.gv.bmi.szr.key.DSAKeyValueType;
import at.gv.bmi.szr.key.KeyValueType;
import at.gv.bmi.szr.key.RSAKeyValueType;
import at.gv.bmi.szr.key.ecdsa.DomainParamsType;
import at.gv.bmi.szr.key.ecdsa.ECDSAKeyValueType;
import at.gv.bmi.szr.key.ecdsa.ECPointType;
import at.gv.bmi.szr.key.ecdsa.NamedCurveType;
import at.gv.bmi.szr.key.ecdsa.PrimeFieldElemType;
import at.gv.bmi.szr.person.DeliveryAddressType;
import at.gv.bmi.szr.person.IdentificationType;
import at.gv.bmi.szr.person.PersonNameType;
import at.gv.bmi.szr.person.PhysicalPersonType;
import at.gv.bmi.szr.person.PostalAddressType;

/**
 * Repository fr Testdaten.
 * <p>
 * Diese Klasse dient zum Bereitstellen von Testdaten fr den
 * SZR Client. Diese Testdaten werden aus einem XML gelesen.
 * (zb. standard-test.xml). Fr dieses XML wurde ein Schema 
 * (standard-test.xsd) definiert.
 * Damit knnen beliebige Test-XML erzeugt werden. Grundstzlich
 * ist diese Datensammlung darauf ausgelegt, die Daten aus 
 * einem XML zu lesen. Es kann jedoch auch ein grundstzlicher
 * Standardfall erzeugt werden.
 * 
 * @author Oliver Ehrenmller
 */
public class TestCasePool {

	/**
	 * SAX-Parser zum Einlesen eines XMLs vom Schema <code>standard-test.xsd</code>. 
	 * Dieser SAX-Parser erzeugt die Dateninstancen und speichert sie in dem
	 * TestCasePool. Dabei werden die Datenklassen vom AxisClient genutzt. Der
	 * Aufbau des XMLs ist aus dem Schema <code>standard-test.xsd</code> nachzulesen.
	 * 
	 * @author Oliver Ehrenmller
	 */
	public class ContentHandlerImpl implements ContentHandler {

		/**
		 * Verweis auf den aktuellen Testfall whrend des Einlesens fr
		 * die richtige Zuweisung neuer Daten. 
		 */
		private TestCase testCase;

		public void characters(char[] arg0, int arg1, int arg2) throws SAXException {
		}

		public void endDocument() throws SAXException {
		}

		/**
		 * Sollte ein Testfall oder eine Operation geschlossen werden, wird die entsprechende
		 * Variable gelscht.
		 * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
		 */
		public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
			if (qName.equals(D.TAG_CASE)) {
				TestCasePool.this.addTestCase(this.testCase);
				this.testCase = null;
			}
		}

		public void endPrefixMapping(String arg0) throws SAXException {
		}

		public void ignorableWhitespace(char[] arg0, int arg1, int arg2) throws SAXException {
		}

		public void processingInstruction(String arg0, String arg1) throws SAXException {
		}

		public void setDocumentLocator(Locator locator) {
		}

		public void skippedEntity(String arg0) throws SAXException {
		}

		public void startDocument() throws SAXException {
		}

		/**
		 * Verarbeitet den Inhalt des XMLs. Organisiert die Struktur und schreibt
		 * die Daten (einlangend als Attribute) in die entsprechenden Datenklassen
		 * vom AxisClient.
		 * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
		 */
		public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException {
			if (qName.equals(D.TAG_CASE)) {
				this.testCase = new TestCase(atts.getValue(D.TESTCASE_NAME));
			} else if (this.testCase != null) {
				if (qName.equals(D.TAG_OPERATION)) {
					String[] array = new String[this.testCase.operations.length + 1];
					System.arraycopy(this.testCase.operations, 0, array, 0, this.testCase.operations.length);
					array[array.length - 1] = atts.getValue(D.OPERATION_NAME);
					this.testCase.operations = array;
				} else if (qName.equals(D.TAG_FAULT)) {
					this.testCase.faultCode = atts.getValue(D.FAULT_CODE);
				} else if (qName.equals(D.TAG_PERSON)) {
					if (this.testCase.personInfo == null)
						this.testCase.personInfo = new PersonInfoType();
					this.testCase.personInfo.setPerson(
						createPerson(atts.getValue(D.PERSON_ZUNAME), atts.getValue(D.PERSON_VORNAME), atts.getValue(D.PERSON_GEBDAT), atts.getValue(D.PERSON_NATION), atts.getValue(D.PERSON_GEBORT), atts.getValue(D.PERSON_GESCHLECHT), atts.getValue(D.PERSON_IDTYPE), atts.getValue(D.PERSON_IDVALUE)));
				} else if (qName.equals(D.TAG_ADDRESS)) {
					if (this.testCase.personInfo == null)
						this.testCase.personInfo = new PersonInfoType();
					this.testCase.personInfo.setRegularDomicile(createAddress(atts.getValue(D.ADDRESS_PLZ), atts.getValue(D.ADDRESS_ORT), atts.getValue(D.ADDRESS_STRASSE), atts.getValue(D.ADDRESS_HAUSNR), atts.getValue(D.ADDRESS_TUER)));
				} else if (qName.equals(D.TAG_DSA)) {
					KeyValueType key = new KeyValueType();
					key.setDSAKeyValue(createDSA(atts.getValue(D.DSA_P), atts.getValue(D.DSA_Q), atts.getValue(D.DSA_G), atts.getValue(D.DSA_Y)));
					this.testCase.key.add(key);
				} else if (qName.equals(D.TAG_RSA)) {
					KeyValueType key = new KeyValueType();
					key.setRSAKeyValue(createRSA(atts.getValue(D.RSA_MOD), atts.getValue(D.RSA_EXP)));
					this.testCase.key.add(key);
				} else if (qName.equals(D.TAG_ECDSA)) {
					KeyValueType key = new KeyValueType();
					key.setECDSAKeyValue(createECDSA(atts.getValue(D.ECDSA_CURVE), atts.getValue(D.ECDSA_X), atts.getValue(D.ECDSA_Y)));
					this.testCase.key.add(key);
				} else if (qName.equals(D.TAG_BPK)) {
					for (int i = 0; i < atts.getLength(); i++) {
						String name = atts.getQName(i);
						if (name.startsWith("target"))
							name = D.BPK_TARGET;
						this.testCase.properties.setProperty(name, atts.getValue(i));
					}
				} else if (qName.equals(D.TAG_SZ)) {
					for (int i = 0; i < atts.getLength(); i++)
						this.testCase.properties.setProperty(atts.getQName(i), atts.getValue(i));
				}
			}
		}

		public void startPrefixMapping(String arg0, String arg1) throws SAXException {
		}

	}

	/**
	 * Sammlung von XML-Tags spezifiert im Schema <code>standard-test.xsd</code>.
	 *
	 * @author Oliver Ehrenmller
	 */
	public static interface D {
		public static final String ADDRESS_HAUSNR = "haus-nr";
		public static final String ADDRESS_ORT = "ort";
		public static final String ADDRESS_PLZ = "plz";
		public static final String ADDRESS_STRASSE = "strasse";
		public static final String ADDRESS_TUER = "tuer";
		public static final String DSA_G = "g";
		public static final String DSA_P = "p";
		public static final String DSA_Q = "q";
		public static final String DSA_Y = "y";
		public static final String ECDSA_CURVE = "curve";
		public static final String ECDSA_X = "x";
		public static final String ECDSA_Y = "y";
		public static final String FAULT_CODE = "code";
		public static final String OPERATION_NAME = "name";
		public static final String TESTCASE_NAME = "name";
		public static final String PERSON_GEBDAT = "geburtsdatum";
		public static final String PERSON_GEBORT = "geburtsort";
		public static final String PERSON_GESCHLECHT = "geschlecht";
		public static final String PERSON_IDTYPE = "id-type";
		public static final String PERSON_IDVALUE = "id-value";
		public static final String PERSON_NATION = "nation";
		public static final String PERSON_VORNAME = "vorname";
		public static final String PERSON_ZUNAME = "zuname";
		public static final String RSA_EXP = "exp";
		public static final String RSA_MOD = "mod";
		public static final String TAG_ADDRESS = "address";
		public static final String TAG_CASE = "case";
		public static final String TAG_DSA = "dsa";
		public static final String TAG_ECDSA = "ecdsa";
		public static final String TAG_FAULT = "fault";
		public static final String TAG_ATTRIBUTES = "attributes";
		public static final String TAG_BPK = "bpk";
		public static final String TAG_SZ = "sz";
		public static final String TAG_OPERATION = "operation";
		public static final String TAG_PERSON = "person";
		public static final String TAG_RSA = "rsa";
		public static final String BPK_BASISZAHL = "basiszahl";
		public static final String BPK_TEST = "bpk-test";
		public static final String BPK_BEREICH = "bereich";
		public static final String BPK_PRODUKTION = "bpk-prod";
		public static final String BPK_TARGET = "target";
		public static final String BPK = "bpk";
		public static final String SZ_TEST = "sz-test";
		public static final String SZ_PROD = "sz-prod";
		public static final String SZ = "sz";
		public static final String ZMR_FREMDBPK = "zmr-fremdbpk";
	}

	public static final String OPERATION_GETBPK = "GETBPK";
	public static final String OPERATION_GETSTAMMZAHL = "GETSTAMMZAHL";
	public static final String OPERATION_TRANSFORMBPK = "TRANSFORMBPK";
	public static final String OPERATION_GETIDENTITYLINK = "GETIDENTITYLINK";
	public static final String OPERATION_BPKZUBASISZAHL = "BPKZUBASISZAHL";
	public static final String OPERATION_BASISZAHLZUBPK = "BASISZAHLZUBPK";
	public static final String OPERATION_ZMRINTEGRATION = "ZMRINTEGRATION";

	/**
	 * Erzeugt eine Adresse aus den bergebenen Werten.
	 * 
	 * @param postalCode Postleitzahl
	 * @param municipality Ort
	 * @param streetName Strasse
	 * @param buildingNumber Hausnummer
	 * @param unit Tr, Stiege
	 * @return die erzeugte Adresse
	 */
	protected static PostalAddressType createAddress(String postalCode, String municipality, String streetName, String buildingNumber, String unit) {
		PostalAddressType postalAddress = new PostalAddressType();
		postalAddress.setPostalCode(postalCode);
		postalAddress.setMunicipality(municipality);
		DeliveryAddressType deliveryAddress = new DeliveryAddressType();
		deliveryAddress.setStreetName(streetName);
		deliveryAddress.setBuildingNumber(buildingNumber);
		deliveryAddress.setUnit(unit);
		postalAddress.setDeliveryAddress(deliveryAddress);
		return postalAddress;
	}

	/**
	 * Erzeugt einen DSA-Schlssel aus den Parametern.
	 * 
	 * @param p		P
	 * @param q		Q
	 * @param g		G
	 * @param y		Y
	 * @return	erzeugter DSA-Schlssel
	 */
	protected static DSAKeyValueType createDSA(String p, String q, String g, String y) {
		DSAKeyValueType dsa = new DSAKeyValueType();
		dsa.setP(p);
		dsa.setQ(q);
		dsa.setG(g);
		dsa.setY(y);
		return dsa;
	}

	/**
	 * Erzeugt einen ECDSA-Schlssel aus den Parametern.
	 * 
	 * @param urn	Bezeichnung der Kurve
	 * @param x		X
	 * @param y		Y
	 * @return	erzeugter ECDSA-Schlssel
	 */
	protected static ECDSAKeyValueType createECDSA(String urn, String x, String y) {
		ECDSAKeyValueType ec = new ECDSAKeyValueType();
		DomainParamsType dp = new DomainParamsType();
		NamedCurveType nc = new NamedCurveType();
		nc.setURN(urn);
		dp.setNamedCurve(nc);
		ec.setDomainParameters(dp);
		ECPointType pt = new ECPointType();
		PrimeFieldElemType elem = new PrimeFieldElemType();
		elem.setValue(x);
		pt.setX(elem);
		elem = new PrimeFieldElemType();
		elem.setValue(y);
		pt.setY(elem);
		ec.setPublicKey(pt);
		return ec;
	}

	/**
	 * Erzeugt eine Person aus den bergebenen Daten.
	 * 
	 * @param familyName	Familienname
	 * @param givenName		Vorname
	 * @param dateOfBirth	Geburtsdatum
	 * @param nationality	Nationalitt
	 * @param placeOfBirth	Geburtsort
	 * @param sex			Geschlecht
	 * @param idType		Id-Typ
	 * @param idValue		Id-Wert (Stammzahl)
	 * @return	erzeugte Person.
	 */
	protected static PhysicalPersonType createPerson(String familyName, String givenName, String dateOfBirth, String nationality, String placeOfBirth, String sex, String idType, String idValue) {

		PhysicalPersonType p = new PhysicalPersonType();
		p.setDateOfBirth(dateOfBirth);
		p.setNationality(nationality);
		p.setPlaceOfBirth(placeOfBirth);
		p.setSex(sex);
		PersonNameType n = new PersonNameType();
		n.setFamilyName(familyName);
		n.setGivenName(givenName);
		p.setName(n);
		IdentificationType it = new IdentificationType();
		it.setType(idType);
		it.setValue(idValue);
		p.setIdentification(it);
		return p;
	}

	/**
	 * Erzeugt eine Personeninfo.
	 * 
	 * @param p Person
	 * @param a Adresse
	 * @return die erzeugte Personeninfo
	 */
	protected static PersonInfoType createPersonInfo(PhysicalPersonType p, PostalAddressType a, TravelDocumentType t) {
		PersonInfoType pi = new PersonInfoType();
		pi.setPerson(p);
		pi.setRegularDomicile(a);
		pi.setTravelDocument(t);
		return pi;
	}

	/**
	 * Erzeugt einen RSA-Schlssel aus den Parametern.
	 * 
	 * @param mod	Modulus
	 * @param exp	Exponent
	 * @return erzeugter RSA-Schlssel
	 */
	protected static RSAKeyValueType createRSA(String mod, String exp) {
		RSAKeyValueType rsa = new RSAKeyValueType();
		rsa.setModulus(mod);
		rsa.setExponent(exp);
		return rsa;
	}

	HashVector cases;
	private PropertiesVector properties;

	/**
	 * Default Constructor.
	 * <p>
	 * Erzeugt ein leeres Testcase Repository.
	 */
	public TestCasePool() {
		this.cases = new HashVector();
		this.properties = new PropertiesVector();
	}

	/*
		public DSAKeyValueType generateSampleDSA() {
			try {
				KeyPairGenerator generator = KeyPairGenerator.getInstance("DSA", "SUN");
				SecureRandom random = SecureRandom.getInstance("SHA1PRNG", "SUN");
				generator.initialize(512, random);
				KeyPair pair = generator.generateKeyPair();
				DSAPublicKey dsa = (DSAPublicKey) pair.getPublic();
				DSAParams params = dsa.getParams();
				return createDSA(
					params.getP().toString(),
					params.getQ().toString(),
					params.getG().toString(),
					dsa.getY().toString());
			} catch (Exception e) {
				e.printStackTrace();
				return null;
			}
		}
		
		*/

	public static RSAPrivateKey sampleRSA() {
		/*
		Schlssel in Base64 
		- Modulus: ALZLJztnLpTSnYxPBr7ecoitDSfogAht529BxjItJuw9mDuTOuuWjDZBpLGu1iMTKIYcCpwntTmSu3tjI+6P9MX3lwV42dDoGaeK26Oj0y7VOAzTLBSCIGuR8cnrnWRO4eykWAfMK2jq8bQ1W1ygBj4xLtaVpoj92SRoFhZhcNfz
		- PublicExponent: AQAB
		- PrivateExponent: AKKACf/9Mdv+h+0MgVHiUC0gPmsF6iQZfMaD1p7h+yF6otrwmwRp1qGDEMJHvaHUsTojgx3Ck903vSL3tI80Cphv6FmHCGbvFzHg/FnEwu+Pl/DTJXSgvp4gg59zitShr7KXfyM1sIznl8P7fOPTubZGux0eW6/tBoSf1sKHjCvR
		- PrimeP: AO493BREHGbStMZWY5rhw/FhcrC1EdJOFxlXCfHFOnh4v+QTMORuBUJmXMxbw+i55gl+yR5kec7hozv+S+Yz1Ok=
		- PrimeQ: AMPhsHdIdIjbf225utkuQXA/gUeVXZF9CQEETadSU/sxGOm3BI2aeJTsV8tuePl+mNKLRDiYs/yhBfMAs111rHs=
		- PrimeExponentP: bLnUd3dCEvb0Q1PnryZ7euyDH/vyvyyeRBVB15GwBozs3hSYCpffVJdMoIL8nHhCcHDpLjKimHTHPzjlwJBFCQ==
		- PrimeExponentQ: ap2x5gr62u7V2gxIhiNmHMmD/UlwR1sOBo6oEZV+/m8Hx1YFVWmpji1hz+lOUUY1ugtxU3jSJoQib9SGzCzbdw==
		- CrtCoefficient: AJKAX1HG8nJJJu9XPOBpKTHN6B0olhPQ4DMoqWEX5QDZtHS1tcwekvqwYho6+3xmG0g40zbd+l1wMtnN0y+avq0=
		*/
		String modulus = "ALZLJztnLpTSnYxPBr7ecoitDSfogAht529BxjItJuw9mDuTOuuWjDZBpLGu1iMTKIYcCpwntTmSu3tjI+6P9MX3lwV42dDoGaeK26Oj0y7VOAzTLBSCIGuR8cnrnWRO4eykWAfMK2jq8bQ1W1ygBj4xLtaVpoj92SRoFhZhcNfz";
		String publicExponent = "AQAB";
		String privateExponent = "AKKACf/9Mdv+h+0MgVHiUC0gPmsF6iQZfMaD1p7h+yF6otrwmwRp1qGDEMJHvaHUsTojgx3Ck903vSL3tI80Cphv6FmHCGbvFzHg/FnEwu+Pl/DTJXSgvp4gg59zitShr7KXfyM1sIznl8P7fOPTubZGux0eW6/tBoSf1sKHjCvR";
		String primeP = "AO493BREHGbStMZWY5rhw/FhcrC1EdJOFxlXCfHFOnh4v+QTMORuBUJmXMxbw+i55gl+yR5kec7hozv+S+Yz1Ok=";
		String primeQ = "AMPhsHdIdIjbf225utkuQXA/gUeVXZF9CQEETadSU/sxGOm3BI2aeJTsV8tuePl+mNKLRDiYs/yhBfMAs111rHs=";
		String primeExponentP = "bLnUd3dCEvb0Q1PnryZ7euyDH/vyvyyeRBVB15GwBozs3hSYCpffVJdMoIL8nHhCcHDpLjKimHTHPzjlwJBFCQ==";
		String primeExponentQ = "ap2x5gr62u7V2gxIhiNmHMmD/UlwR1sOBo6oEZV+/m8Hx1YFVWmpji1hz+lOUUY1ugtxU3jSJoQib9SGzCzbdw==";
		String crtCoefficient = "AJKAX1HG8nJJJu9XPOBpKTHN6B0olhPQ4DMoqWEX5QDZtHS1tcwekvqwYho6+3xmG0g40zbd+l1wMtnN0y+avq0=";
		return new RSAPrivateKey(toBigInteger(modulus), toBigInteger(publicExponent), toBigInteger(privateExponent), toBigInteger(primeP), toBigInteger(primeQ), toBigInteger(primeExponentP), toBigInteger(primeExponentQ), toBigInteger(crtCoefficient));
	}

	public static BigInteger toBigInteger(String base64) {
		return new BigInteger(toByteArray(base64));
	}

	public static byte[] toByteArray(String base64) {
		try {
			byte[] bytes = new byte[(base64.length() / 4) * 3 + 2];
			ByteArrayInputStream bis = new ByteArrayInputStream(base64.getBytes());
			Base64InputStream is = new Base64InputStream(bis);
			int number = is.read(bytes);
			is.close();
			byte[] result = new byte[number];
			System.arraycopy(bytes, 0, result, 0, number);
			return result;
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
	}

	public static String toBase64(byte[] array) {
		try {
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			Base64OutputStream os = new Base64OutputStream(bos);
			os.write(array);
			os.close();
			return bos.toString();
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
	}

	public boolean containsProperty(String operation, String name) {
		return this.properties.containsKey(operation + "_" + name);
	}

	public String getProperty(String operation, String name) {
		return this.properties.getProperty(operation + "_" + name);
	}

	public String getProperty(String operation, String name, String defaultValue) {
		return this.properties.getProperty(operation + "_" + name, defaultValue);
	}

	/**
	 * Liefert alle Testflle fr einen bestimmten Testbereich
	 * @param testBereich Testbereich (zb. BPK)
	 * @return alle Testflle eines Testbereiches
	 */
	public Iterator getTestCases(String testBereich) {
		return this.cases.getAll(testBereich).iterator();
	}

	/**
	 * Liefert einen bestimmten Testfall fr einen Testbereich, angegeben
	 * durch seinen Array-Index 
	 * @param testBereich	TestBereich (zb. BPK)
	 * @param index			Index des Testbereiches.
	 * @return ein bestimmter Testfall eines Testbereiches
	 */
	public TestCase getTestCase(String testBereich, int index) {
		return (TestCase) this.cases.getAll(testBereich).elementAt(index);
	}

	/**
	 * Gibt die Anzahl der Testflle fr einen Testbereich an.
	 * @param testBereich	TestBereich (zb. BPK)
	 * @return Anzahl der Testflle.
	 */
	public int getTestCaseCount(String testBereich) {
		return this.cases.getAll(testBereich).size();
	}

	/**
	 * Fllt das Repository aus einer Datei.
	 * 
	 * @param fileName Name der zu lesenden Datei
	 * @throws Exception wenn ein Fehler auftritt
	 */
	public void load(String fileName) throws SAXException, ParserConfigurationException, IOException {
		ContentHandlerImpl ch = new ContentHandlerImpl();
		SAXParserFactory factory = SAXParserFactory.newInstance();
		SAXParser parser = factory.newSAXParser();
		XMLReader reader = parser.getXMLReader();
		reader.setContentHandler(ch);
		reader.parse(fileName);
	}

	/**
	 * Fgt den Testfall dem Pool hinzu unter Bercksichtigung der Operationen
	 * @param testCase neuer Testfall
	 */
	private void addTestCase(TestCase testCase) {
		for (int i = 0; i < testCase.operations.length; i++)
			this.cases.put(testCase.operations[i], testCase);
	}

	/**
	 * Fllt das Repository mit fixen Testdaten.
	 */
	public void loadDefaults() {
		TestCase testCase;
		PhysicalPersonType person;
		PostalAddressType address;
		KeyValueType key;

		/*
		 * XXXMuster, XXXMax, 1.1.1980
		 * XXXBerger, XXXJosef, 18.06.1951 (nicht eindeutig)
		 * XXXWeber, XXXMelanie, 1.1.1950
		 */
		testCase = new TestCase("Standard-Test");
		testCase.operations = new String[] { OPERATION_BPKZUBASISZAHL, OPERATION_GETBPK, OPERATION_GETIDENTITYLINK, OPERATION_TRANSFORMBPK, OPERATION_BASISZAHLZUBPK, OPERATION_ZMRINTEGRATION, OPERATION_GETSTAMMZAHL };

		person = createPerson("XXXMuster", "XXXMax", "1980-01-01", null, null, null, null, null);
		//person = createPerson("XXXWeber", "XXXMelanie", "1950-01-01", null, null, null, null, null);
		//person = createPerson("XXXBerger", "XXXJosef", "1951-06-18", null, null, null, null, null);
		//person = createPerson("XXXMuster", "XXXMax", "1980-01-01", null, null, "male", "urn:publicid:gv.at:cdid+TEST", "nwQtFJBBvrbNwCOGHO1GkUE6ry4=");
		//person = createPerson("XXXOttakringer", "", "1973-01-01", null, null, "male", "urn:publicid:gv.at:cdid+SV", "pzQVv6NKXYN6aVLKwQ0L7CQ8Gh8=");
		//person = createPerson("XXXPurgstallderFamilienname-ist-genau-vierundsechzigZeichen-lang", "XXXika-Angelika-Christine-Ccile-Annemarie-Friederike-Armelnde", "1965-10-12", null, null, null, null, null);
		//person = createPerson("XXXPurgstallderFamilienname-ist-genau-vierundsechzigZeichen-lang", "", "1965-10-12", null, null, "female", null, null);
		//person = createPerson("XXX?d?", "XXXLudmilla", "1965-10-12", null, null, "", null, null);
		//person = createPerson("XXXvon Brandenburg", "XXXClaus-Maria", "1984-12-31", null, null, null, null, null);

		address = new PostalAddressType();
		DeliveryAddressType delad = new DeliveryAddressType();
		TravelDocumentType travel = new TravelDocumentType();
		delad.setStreetName(""); // Leer-Text als Test wegen NullPointerException-Problem
		address.setDeliveryAddress(delad);
		address.setPostalCode("");
		//address.setMunicipality("Wien");
		testCase.personInfo = createPersonInfo(person, address, travel);
		//testCase.personInfo = createPersonInfo(person, null, null);

		key = new KeyValueType();
		RSAPrivateKey rsa = sampleRSA();
		key.setRSAKeyValue(createRSA(toBase64(rsa.getModulus().toByteArray()), toBase64(rsa.getPublicExponent().toByteArray())));
		testCase.key.add(key);
		key = new KeyValueType();
		key.setDSAKeyValue(createDSA("xxz6", "qqq", "ggg", "yyyy"));
		testCase.key.add(key);
		key = new KeyValueType();
		key.setECDSAKeyValue(createECDSA("urn:oid:1.2.840.10045.3.1.1", "5922595088151856706638392127820825146781740943019774201480", "3333077238634515163826912101083366752245521657747266021259"));
		testCase.key.add(key);

		testCase.properties.setProperty(D.BPK_BEREICH, "urn:publicid:gv.at:cdid+TEST");
		//testCase.properties.setProperty(D.BPK_BEREICH, "urn:publicid:gv.at:cdid+OW");
		//testCase.properties.setProperty(D.BPK_BEREICH, "urn:publicid:gv.at:cdid+SV");
		//testCase.properties.setProperty(D.BPK_BEREICH, "urn:publicid:gv.at:cdid+OW");
		testCase.properties.setProperty(D.BPK_TARGET, "urn:publicid:gv.at:cdid+T1");
		testCase.properties.setProperty(D.BPK_TARGET, "urn:publicid:gv.at:cdid+T2");

		//testCase.properties.setProperty(D.BPK_TEST, "Onsb/JY2eiJSQttLbCtqspaQqMk=");

		// Daten fr Muster Max
		testCase.properties.setProperty(D.BPK_BASISZAHL, "000531128185");
		testCase.properties.setProperty(D.BPK_BASISZAHL, "000314395555"); // falsch
		testCase.properties.setProperty(D.BPK_BASISZAHL, "000314395555"); // falsch
		testCase.properties.setProperty(D.BPK_BASISZAHL, "000314395555"); // falsch

		//testCase.properties.setProperty(D.BPK_TEST, "SsocmwVv/9JxnekMJ5m8haz0XLU="); // local,wsad
		testCase.properties.setProperty(D.BPK_TEST, "QMnF5BMqFqf9pZ/6UKnoPA48M8g=");
		testCase.properties.setProperty(D.BPK_PRODUKTION, "3qy6gbqj+4vHCQvhOrYhZUGh35M=");

		testCase.properties.setProperty(D.SZ_TEST, "LowHNpoxSU0oCecVZmddFQ==");
		testCase.properties.setProperty(D.SZ_PROD, "RzLLN4RN6B/iMzKGqSds5A==");
		
		testCase.properties.setProperty(D.ZMR_FREMDBPK, "Lv9OxL5qc4B1T2HQthSc7kZ8za28DteYm9FTNJc/SQwkEJlhzlJTeNkYIlAdQfz7om0GPMr9i2mkvpfITLktUgZe97gl2XY9Dyx2CE5h6MGavOpj6HHv7aijyVa7C29BOaytY2c27efHIVN6dcVXutJPERJFZPUfHOl1DHfmZ/A=");
		testCase.properties.setProperty(D.ZMR_FREMDBPK, "GPEfKE++32T+qQ2rAkYdfcnlkrO0Q7X10V0tJp1gChm9DtN1lifWIOlEn7/l8yxg7DLcrRFRemz0AoB9nOc7oisrA/0WY53z9L7Ju0BsrPiq30FvCYdp5LLeAW0cYebkVsC2G/o+q6QTzEF++yafXElWSZ1mdvIiy7iFG/CMCXw=");
		testCase.properties.setProperty(D.ZMR_FREMDBPK, "HfGP95/ASUVYXhULIwbWB+K92wtXNwqd8vd8UiCYqdEt2XNJ9L3ExJ+nSe3RWnMzZgi5mv6qawrylr8zrtc7Xp65oTzkQu8lzm317RgbWfym8i/PjiTExVNIpXS/Dnqr3cfw+XSit1daxrqAUJ7KQce9JWN9vGtjb2ELXzfwfrM=");
		testCase.properties.setProperty(D.ZMR_FREMDBPK, "Cx7jJbdjJ1X4ECWqRgi+wp08OW4CffwHqd2aO6cIhzsSWqvF9J8AjlIZzbUI8gTe3LdvWQ/EMOWhUfEIP9z8eQw88FpaYLfUk3HS5HKSE9fwZ+iToGL/BqloxKkNvzLbvZLHTB+gsscAqa8hS1aK0nyQDntcW9ovcfp82EMzIwA=");
		testCase.properties.setProperty(D.ZMR_FREMDBPK, "S3tHgEauo1MsgSqC2fIlkD7QkKyRqV8Gyusq1yEDhtWgVWO41TZz4ZSpHJTT0JCl+abwHtKM6zpIJ09hoHCjIcotDrLTleGVmz4sLTP2/RcWbMctZCjTcy0QkiwByYm6AzYngz6EQXNv3hk45m0pp9Qv+gOVJSv9ShsCIABmRrU=");
		testCase.properties.setProperty(D.ZMR_FREMDBPK, "ePPpFwIGbmUK2Csl4XkYtlZIoiTtRxUDe4mRgdynAgJK75NVl2mc7/Fqbw+e5wOba7IZICatJmMm8fmNv/t7RIMZjKC2G8qjSgnrc3PPQ/FHK6mDaoqDgU5BRSyfsRdpyhxTWYK/PeaOLemqJEz9IZMJ9DzpFS3xj4wSs7Vk+zo=");

		this.addTestCase(testCase);
	}

}
