package moazs.api.dn;

import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import moazs.Constants;
import moazs.api.MoaException;
import moazs.api.dn.impl.LogDeliveryNotificationHandler;
import moazs.external.MoaSP;
import moazs.util.MoaConfig;
import moazs.util.MoaXMLUtils;
import moazs.util.MoaXPath;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import at.gv.egovernment.moa.util.DOMUtils;

public class DeliveryNotificationServiceHandler {

  static private Logger logger = Logger.getLogger(DeliveryNotificationServiceHandler.class);
  
  private MoaConfig config = null;
  
  private String NS_MZS = null;
  private String NS_ZUS = null;
  
  public DeliveryNotificationServiceHandler() {
    config = new MoaConfig();
    this.NS_ZUS = config.getProperty("zus", "Namespaces");
    this.NS_MZS = config.getProperty("mzs", "Namespaces");
  }
  
  public Document DeliveryNotification(Document in) {
    
    boolean isActive = config.getDeliveryNotificationBooleanProperty("active");
    if (!isActive) {
      logger.info("Delivery Notification Service not active!! Incoming delivery notification request not handled.");
      return null;
    }
    
    logger.info("MOA-ZS notification request.");
    
    // debug incoming documents
    if (config.getDebugRequests()) {
      try {
        logger.debug(MoaXMLUtils.serializeToString(in.getDocumentElement(), true));
      } catch (IOException e1) {
        logger.warn("Error writing incoming request.", e1);
      }
    }
    
    try {
      // validate xml document
      logger.debug("Schema validation of notification: " + config.getSchemaValidation());
      
      // check schema validation
      if (config.getSchemaValidation()) {
        try {
          MoaXMLUtils.validateZUSE(in.getDocumentElement());
          logger.debug("Schema validation successful.");
        } catch (Throwable e) {
          logger.debug("Schema validation failed.", e);
          throw new MoaException(Constants.ERROR_REQUEST_SCHEMA_VALIDATION_FAILED, "Request schema validation failed.");
        }
      }
      
      /*
       * ZUSE delivery notification (RSa)
       */
      if (NS_ZUS.equals(in.getDocumentElement().getNamespaceURI()) && Constants.DELIVERYNOTIFICATION.equals(in.getDocumentElement().getLocalName())) {
        logger.debug("Handling ZUSE delivery notification.");
        // we have a ZUSE notification
        // check for zuse success or error
        Element success = MoaXPath.getFirstElement(in.getDocumentElement(), "/zus:DeliveryNotification/zus:Success");
        String signatureLocation = "";
        // success
        if (success != null) {
          signatureLocation = "/zus:DeliveryNotification/zus:Success/dsig:Signature";
        // error
        } else {
          signatureLocation = "/zus:DeliveryNotification/zus:Error/zuse:ZUSESignature";
        }
        // verify delivery notification with moa-sp
        if (config.verifyNotificationSignature()) {
          logger.debug("Verifying signature of delivery notification.");
          MoaSP moaSP = new MoaSP();
          if (!moaSP.verify(in.getDocumentElement(), signatureLocation)) {
            throw new MoaException(Constants.ERROR_MOASP_SIGNATURE_INVALID, "Signature of delivery notification is invalid.");
          }
        } else {
          logger.debug("Skipping signature verification of delivery notification.");
        }
        
        // parse AppDeliveryId and ZSDeliveryId
        String appDeliveryId = MoaXPath.getFirstString(in.getDocumentElement(), "//zus:AppDeliveryID/text()");
        String zsDeliveryId = MoaXPath.getFirstString(in.getDocumentElement(), "//zus:ZSDeliveryID/text()");
        
        // handle delivery notification
        // get implementation
        String className = config.getDeliveryNotificationProperty("plugin");
        // default plugin
        if (className == null || "".equals(className)) {
          LogDeliveryNotificationHandler handler = new LogDeliveryNotificationHandler();
          handler.handleZUSEDeliveryNotification(in.getDocumentElement());
        // try to dynamically load plugin
        } else {
          //logger.debug("Trying to load plugin " + className);
          try {
            Class pluginClass = Class.forName(className);
            Object newInstance = pluginClass.newInstance();
            if (! (newInstance instanceof DeliveryNotificationHandler)) {
              logger.warn("Cannot load plugin: class " + className + " not instanceof " + DeliveryNotificationHandler.class.getName());
            } else {
              DeliveryNotificationHandler notHandler = (DeliveryNotificationHandler) newInstance;
              notHandler.handleZUSEDeliveryNotification(in.getDocumentElement());
            }
          } catch (ClassNotFoundException e) {
            logger.warn("Error loading delivery notification plugin: class not found (" + className + ")");
          } catch (InstantiationException e) {
            logger.warn("Error loading delivery notification plugin [" + className + "]: " + e.getMessage());
          } catch (IllegalAccessException e) {
            logger.warn("Error loading delivery notification plugin [" + className + "]: " + e.getMessage());
          } 
        }
        // send delivery notification ACK
        return createDeliveryNotificationACK(appDeliveryId, zsDeliveryId);


      /*
       *  MOA-ZS delivery notification
       */
      } else if (NS_MZS.equals(in.getDocumentElement().getNamespaceURI()) && Constants.DELIVERYNOTIFICATION.equals(in.getDocumentElement().getLocalName())) {
        logger.debug("Handling MZS delivery notification.");
        
        // parse AppDeliveryId and ZSDeliveryId
        //String appDeliveryId = MoaXPath.getFirstString(in.getDocumentElement(), "//zus:AppDeliveryID/text()");
        //String zsDeliveryId = MoaXPath.getFirstString(in.getDocumentElement(), "//zus:ZSDeliveryID/text()");
        
        // handle delivery notification
        // get implementation
        String className = config.getDeliveryNotificationProperty("plugin");
        // default plugin
        if (className == null || "".equals(className)) {
          LogDeliveryNotificationHandler handler = new LogDeliveryNotificationHandler();
          handler.handleZUSEDeliveryNotification(in.getDocumentElement());
        // try to dynamically load plugin
        } else {
          //logger.debug("Trying to load plugin " + className);
          try {
            Class pluginClass = Class.forName(className);
            Object newInstance = pluginClass.newInstance();
            if (! (newInstance instanceof DeliveryNotificationHandler)) {
              logger.warn("Cannot load plugin: class " + className + " not instanceof " + DeliveryNotificationHandler.class.getName());
            } else {
              DeliveryNotificationHandler notHandler = (DeliveryNotificationHandler) newInstance;
              notHandler.handleMZSDeliveryNotification(in.getDocumentElement());
            }
          } catch (ClassNotFoundException e) {
            logger.warn("Error loading delivery notification plugin: class not found (" + className + ")");
          } catch (InstantiationException e) {
            logger.warn("Error loading delivery notification plugin [" + className + "]: " + e.getMessage());
          } catch (IllegalAccessException e) {
            logger.warn("Error loading delivery notification plugin [" + className + "]: " + e.getMessage());
          } 
        }
        return null;
      } else {
        throw new MoaException(Constants.ERROR_PROCESSING_UNCLASSIFIED, "Unknown messsage received: " + in.getDocumentElement().getLocalName() + "[" + in.getNamespaceURI() + "]");
      }
      
    } catch(MoaException e) {
      logger.warn("Error handling delivery notification.", e);
      return null;
    } catch(ParserConfigurationException e) {
      logger.warn("Error handling delivery notification.", e);
      return null;
    }
    
    
    
  }
  
  // create delivery notification ACK
  private Document createDeliveryNotificationACK(String appDeliveryId, String zSDeliveryId) throws ParserConfigurationException {
    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
    dbf.setNamespaceAware(true);
    DocumentBuilder db = dbf.newDocumentBuilder();
    Document retDoc = db.getDOMImplementation().createDocument(NS_ZUS, "zus:DeliveryNotificationACK", null);
    retDoc.getDocumentElement().setAttribute("xmlns:zus", NS_ZUS);
    
    Element deliveryNotificationACKElement = retDoc.getDocumentElement();
    
    // AppDeliveryId
    Element appDeliveryIdElement = retDoc.createElementNS(NS_ZUS, "zus:AppDeliveryID");
    appDeliveryIdElement.appendChild(retDoc.createTextNode(appDeliveryId));
    deliveryNotificationACKElement.appendChild(appDeliveryIdElement);
    
    // ZSDeliveryId
    Element zsDeliveryIdElement = retDoc.createElementNS(NS_ZUS, "zus:ZSDeliveryID");
    zsDeliveryIdElement.appendChild(retDoc.createTextNode(zSDeliveryId));
    deliveryNotificationACKElement.appendChild(zsDeliveryIdElement);
    
    return retDoc;
  }
  
  
}
