/*
 * Copyright 2004 Bundeskanzleramt sterreich
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package moazs.external;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.rmi.RemoteException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.Properties;
import java.util.Set;

import javax.activation.DataHandler;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.rpc.ServiceException;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPMessage;

import moazs.Constants;
import moazs.api.MoaException;
import moazs.modell.Payload;
import moazs.modell.Postaladdress;
import moazs.modell.Request;
import moazs.modell.Zuseserver;
import moazs.util.ByteArrayDataSource;
import moazs.util.MoaConfig;
import moazs.util.MoaXMLUtils;

import org.apache.axis.SOAPPart;
import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.message.SOAPBodyElement;
import org.apache.axis.message.SOAPEnvelope;
import org.apache.axis.soap.MessageFactoryImpl;
import org.apache.log4j.Logger;
import org.hibernate.classic.Session;
import org.w3c.dom.DOMException;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import at.gv.egovernment.moa.util.DOMUtils;


/**
 * Klasse zur Uebergabe an den Zustelldienst
 * 
 */
public class MoaDelivery {

    private static Logger logger = Logger.getLogger(MoaDelivery.class);

    private static String NS_P;
    private static String NS_NS1;
    private static String NS_XSI;
                    
    // either zbPK or edID (in private delivery case)
    private String uniqueIdentifier = null;
    private Request req = null;

    private MoaConfig config;
    
    /**
     * Konstruktor
     */
    public MoaDelivery() {
        this.config = new MoaConfig();

        this.NS_NS1		  = this.config.getProperty("zus",	 "Namespaces");
        this.NS_P		  = this.config.getProperty("p1",	 "Namespaces");
        this.NS_XSI       = this.config.getProperty("xsi",	 "Namespaces");
    }

    /**
     * Erstellen des DeliveryRequests an den Zustelldienst
     * 
     * @param sess
     *            Session
     * @param r
     *            Request
     * @return Rueckgabewert -1 bei fehlender Verbindung zum Zustelldienst
     */
    public int createDeliveryRequest(Session sess, Request r) {
        logger.debug("Creating delivery request.");

        req = r;
        Zuseserver zs = req.getZuseserver();
        uniqueIdentifier = zs.getBpk();
        String profileId = req.getSenderProfileId();

        // create delivery document
        Document doc = null;
        try {
          doc = createDeliveryDocument(profileId, r.isPrivateDelivery());
        } catch(MoaException e) {
          return -1;
        }

        // debug request for delivery server
        if (config.getDebugRequests()) {
          try {
            logger.debug(MoaXMLUtils.serializeToString(doc.getDocumentElement(), true));
          } catch (IOException e) {
            logger.warn("Error while debugging delivery request.", e);
          }
        }
        
        // check for demo mode
        if (config.isDemoMode()) {
          logger.info("Demo mode active - Skipping sending of delivery request to delivery server: " + req.getZuseserver().getUrl());
          AppNotification appn = new AppNotification(r.getSenderProfileId(), r.getAppDeliveryId() , r.getSyncId());
          appn.sendSuccess(String.valueOf(r.getId()));
          return 4;
        }
        
        //SOAP Document erstellen
        SOAPBodyElement soapBE = new SOAPBodyElement(doc.getDocumentElement());

        MessageFactory mf = null;
        SOAPMessage sm = null;
        SOAPPart sp = null;
        SOAPEnvelope env = null;
        try {
            mf = new MessageFactoryImpl();
            sm = mf.createMessage();
            
            sp = (SOAPPart) sm.getSOAPPart();
            env = (SOAPEnvelope) sp.getEnvelope();
            env.addNamespaceDeclaration("p", NS_P);
            env.addNamespaceDeclaration("zs", NS_NS1);
            env.addBodyElement(soapBE);
        } catch (Exception e) {
            logger.warn("Error while creating SOAP Envelope", e);
            return -1;
        }


        Service service = new Service();
        try {
            Call scall = (Call) service.createCall();
            logger.debug("URL delivery service: " + req.getZuseserver().getUrl()); 
            scall.setTargetEndpointAddress(req.getZuseserver().getUrl());
            scall.setProperty(Call.ATTACHMENT_ENCAPSULATION_FORMAT, Call.ATTACHMENT_ENCAPSULATION_FORMAT_MIME);
            
            Payload pl = (Payload) req.getPayload().iterator().next();

            DataHandler dh = new DataHandler(new ByteArrayDataSource(pl.getDataAsByteArray(), pl.getMimetype(), pl.getFilename()));

            scall.addAttachmentPart(dh);
//            sess.disconnect();
            SOAPEnvelope rEnv =  null;
            try {
              rEnv = scall.invoke(env);
            } catch(Exception e) {
              e.printStackTrace();
              return -1;
            }
            SOAPBodyElement sbe = rEnv.getFirstBody();
//	    sess.reconnect();

            Document response_doc = sbe.getAsDocument();
            AppNotification appn = new AppNotification(req.getSenderProfileId(), req.getAppDeliveryId() , req.getSyncId());

            if (response_doc.getElementsByTagNameNS(NS_NS1, "Success") != null
                    && response_doc.getElementsByTagNameNS(NS_NS1, "Success").getLength() != 0) {
                logger.debug("Delivery to server successful.");
                //success -> delivery respones an application senden

                appn.sendSuccess(String.valueOf(req.getId()));
                return 1;
                //successfull delete the request and we are done
            } else {
                logger.debug("Error returned by delivery server.");
                if (response_doc.getElementsByTagNameNS(NS_NS1, "Code") != null) {
                    String errcode = response_doc.getElementsByTagNameNS(NS_NS1, "Code").item(0).getFirstChild()
                            .getNodeValue();
                    String errmsg = response_doc.getElementsByTagNameNS(NS_NS1, "Text").item(0).getFirstChild()
                            .getNodeValue();

                    appn.sendError(new String().valueOf(req.getId()), errcode, "Error vom Zustelldienst ("+errmsg+")", "referenz");
                }
                return 0;
                //error get errorcode and handle
            }
        } catch (ServiceException se) {
            logger.warn("Fehler, kann Service Call nicht erzeugen", se);
            return -1;
        } catch (RemoteException re) {
            logger.warn("Fehler, keine Verbindung zum Zuseserver, spaeter nochmal versuchen", re);
            return -1;
        } catch (Exception e) {
           logger.warn( "exception", e);
           return -1;
        }
    }

    /**
     * erstellt das DOM Dokument
     * 
     * @param profileId
     *            Die Applikationskennung (Senderinformation)
     * @return Dokument fuer die Anfrage an den Zustelldienst
     * @throws MoaException 
     */
    private Document createDeliveryDocument(String profileId, boolean isPrivateDelivery) throws MoaException {
        Document doc = null;
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            DocumentBuilder builder = factory.newDocumentBuilder();
            DOMImplementation impl = builder.getDOMImplementation();

            // Create the document
            doc = impl.createDocument(NS_NS1, "DeliveryRequest", null);
            Element root = doc.getDocumentElement();
            
            // set version attribute
            root.setAttribute("version", Constants.APP2ZUSE_VERSION);
            
            //namespaces are added on the soap envelope
            root.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns", NS_NS1);
            root.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:p", NS_P);
            root.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:xsi", NS_XSI);

            // check if private delivery (gvZbpk <-> edID)
            Element uniqueIdentifierElement;
            if (isPrivateDelivery) {
              uniqueIdentifierElement = doc.createElementNS(NS_NS1, "edID");
            } else {
              uniqueIdentifierElement = doc.createElementNS(NS_NS1, "ZbPK");
            }
            uniqueIdentifierElement.appendChild(doc.createTextNode(uniqueIdentifier));

            root.appendChild(uniqueIdentifierElement);
            
            doc = addSender(doc, profileId,isPrivateDelivery);
            doc = addReceiver(doc);
            doc = addMetaData(doc);
        } catch (FactoryConfigurationError fe) {
            logger.warn("FactoryConfigurationError while creating delivery document.", fe);
            throw new MoaException(Constants.ERROR_PROCESSING_UNCLASSIFIED, "Unclassified error while creating delivery document.");
        } catch (ParserConfigurationException pe) {
            logger.warn("ParserConfigurationError while creating delivery document.", pe);
            throw new MoaException(Constants.ERROR_PROCESSING_UNCLASSIFIED, "Unclassified error while creating delivery document.");
        } catch (DOMException de) {
            logger.warn("DOMException  while creating delivery document.", de);
            throw new MoaException(Constants.ERROR_PROCESSING_UNCLASSIFIED, "Unclassified error while creating delivery document.");
        }
        return doc;
    }

    /**
     * fuegt die Empfaengerinformationen ein (nur zur Information) der Zustelldienst verwendet ausschliesslich die zvbPK
     * fuer Zustellung
     * 
     * @param doc
     *            DOM Dokument
     * @return um die Empfaengerinformationen erweitertes DOM Dokument
     */
    private Document addReceiver(Document doc) {

        Element root = doc.getDocumentElement();

        Element receiver = doc.createElementNS(NS_NS1, "Receiver");

        if (req.getRcpt().getPhysicalPerson()) {
            Element person = doc.createElementNS(NS_P, "p:PhysicalPerson");
            Element name = doc.createElementNS(NS_P, "p:Name");
            Element vname = doc.createElementNS(NS_P, "p:GivenName");
            vname.appendChild(doc.createTextNode(req.getRcpt().getGivenname()));
            Element nname = doc.createElementNS(NS_P, "p:FamilyName");
            nname.appendChild(doc.createTextNode(req.getRcpt().getFamilyname()));

            name.appendChild(vname);
            name.appendChild(nname);

            person.appendChild(name);

            Date bd = req.getRcpt().getDateofbirth();
            if (bd != null) {
                Element birthday = doc.createElementNS(NS_P, "p:DateOfBirth");
                SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd'-02:00'");
                birthday.appendChild(doc.createTextNode(fmt.format(req.getRcpt().getDateofbirth())));
                person.appendChild(birthday);
            }

            receiver.appendChild(person);

        } else {
            Element company = doc.createElementNS(NS_P, "p:CorporateBody");
            Element fname = doc.createElementNS(NS_P, "p:FullName");
            fname.appendChild(doc.createTextNode(req.getRcpt().getFullname()));

            company.appendChild(fname);
            receiver.appendChild(company);
        }

        Set pA = req.getRcpt().getPostaladdress();
        for (Iterator i = pA.iterator(); i.hasNext();) {
            Postaladdress postal = (Postaladdress) i.next();
            Element postalAddress = doc.createElementNS(NS_P, "p:PostalAddress");
            Element postalCode = doc.createElementNS(NS_P, "p:PostalCode");
            postalCode.appendChild(doc.createTextNode(postal.getPostalcode()));
            Element municipality = doc.createElementNS(NS_P, "p:Municipality");
            municipality.appendChild(doc.createTextNode(postal.getMunicipality()));
            Element deliveryAddress = doc.createElementNS(NS_P, "p:DeliveryAddress");
            Element streetName = doc.createElementNS(NS_P, "p:StreetName");
            streetName.appendChild(doc.createTextNode(postal.getStreetname()));
            Element buildingNr = doc.createElementNS(NS_P, "p:BuildingNumber");
            buildingNr.appendChild(doc.createTextNode(postal.getBuilding()));

            deliveryAddress.appendChild(streetName);
            deliveryAddress.appendChild(buildingNr);

            postalAddress.appendChild(postalCode);
            postalAddress.appendChild(municipality);
            postalAddress.appendChild(deliveryAddress);

            receiver.appendChild(postalAddress);
        }

        root.appendChild(receiver);
        return doc;
    }

    /**
     * fuegt die notwendigen Metadaten ein
     * 
     * @param doc
     *            DOM Dokument
     * @return um die Metadaten erweitertes DOM Dokument
     * @throws MoaException 
     */
    private Document addMetaData(Document doc) throws MoaException {
        MoaConfig mcfg = new MoaConfig();
        String deliverB = mcfg.getProperty("deliverBefore", "MoaDelivery");
        String notificationB = mcfg.getProperty("notificationBefore", "MoaDelivery");
        int deliverOffset = 0;
        int notificationOffset = 0;
        try {
            notificationOffset = new Integer(notificationB).intValue();
            deliverOffset = new Integer(deliverB).intValue();
        } catch (NumberFormatException e1) {
            logger.warn("Fehler beim Lesen der Offsetwerte fuer DeliverBefore und StartNotificationBefore");
        }

        

        Element root = doc.getDocumentElement();
        try {
            SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS");
            Calendar deliverDate = Calendar.getInstance();
            deliverDate.add(Calendar.HOUR_OF_DAY, deliverOffset);
            String delDate = fmt.format(deliverDate.getTime());
            Calendar notificationDate = Calendar.getInstance();
            notificationDate.add(Calendar.HOUR_OF_DAY, notificationOffset);
            String notDate = fmt.format(notificationDate.getTime());
            Element metaData = doc.createElementNS(NS_NS1, "MetaData");
            Element appDeliveryID = doc.createElementNS(NS_NS1, "AppDeliveryID");
            appDeliveryID.appendChild(doc.createTextNode(req.getAppDeliveryId()));
            Element mzsDeliveryID = doc.createElementNS(NS_NS1, "MZSDeliveryID");
            mzsDeliveryID.appendChild(doc.createTextNode(new Long(req.getId()).toString()));
            Element deliveryQuality = doc.createElementNS(NS_NS1, "DeliveryQuality");
            deliveryQuality.appendChild(doc.createTextNode(req.getDeliveryQuality().getName()));
            
            
            
            Element startNotification = doc.createElementNS(NS_NS1, "StartNotificationBefore");
            startNotification.appendChild(doc.createTextNode(notDate));
            Element deliverBefore = doc.createElementNS(NS_NS1, "DeliverBefore");
            deliverBefore.appendChild(doc.createTextNode(delDate));

            metaData.appendChild(appDeliveryID);
            metaData.appendChild(mzsDeliveryID);
            metaData.appendChild(deliveryQuality);
            // document class for private delivery
            if (req.getDocumentClass() != null) {
              Element documentClass = doc.createElementNS(NS_NS1, "DocumentClass");
              documentClass.appendChild(doc.createTextNode(req.getDocumentClass()));
              metaData.appendChild(documentClass);
            }
            metaData.appendChild(startNotification);
            metaData.appendChild(deliverBefore);  
            
            // check for dual delivery - zbpk is empty only in case of dual delivery
            if ("".equals(req.getZuseserver().getBpk())) {
              if (req.getDualDelivery() != null && req.getDualDelivery().getPrintParameter() != null) {
                try {
                  byte[] printParameterBytes = req.getDualDelivery().getPrintParameterAsByteArray();
                  Document ppDoc = DOMUtils.parseDocument(new ByteArrayInputStream(printParameterBytes), false, null, null);
                  Element printParameter = doc.createElementNS(NS_NS1, "PrintParameter");
                  
                  Element ppRoot = ppDoc.getDocumentElement();
                  if (ppRoot != null) {
                    // import PrintParameter child elements
                    NodeList printParameterContent = ppRoot.getChildNodes();
                    for (int i=0; i<printParameterContent.getLength(); i++) {
                      printParameter.appendChild(doc.importNode(printParameterContent.item(i), true));
                    }
                  }
                  
                  metaData.appendChild(printParameter);
                } catch(IOException e) {
                  logger.warn("Error reading print parameters from database.", e);
                  throw new MoaException(Constants.ERROR_DUAL_UNCLASSIFIED, "Error reading print parameters from database.");
                } catch (SAXException e) {
                  logger.warn("Error reading print parameters from database.", e);
                  throw new MoaException(Constants.ERROR_DUAL_UNCLASSIFIED, "Error reading print parameters from database.");
                } catch (ParserConfigurationException e) {
                  logger.warn("Error reading print parameters from database.", e);
                  throw new MoaException(Constants.ERROR_DUAL_UNCLASSIFIED, "Error reading print parameters from database.");
                }
              }
            }

            root.appendChild(metaData);

        } catch (DOMException e) {
            logger.warn("DOMException beim erstellen der MetaDaten");
        } 
        return doc;
    }

    /**
     * fuegt die Senderinformationen ein
     * 
     * @param doc
     *            DOM Dokument
     * @param profileId
     *            Applikationskennung
     * @param privateVersender
     * 				
     * 
     * @return das um die Senderinformationen erweiterte DOM Dokument
     */
    public Document addSender(Document doc, String profileId,boolean privateVersender) {
        MoaConfig mcfg = new MoaConfig();
        Properties senderProb = mcfg.getSenderInfo(profileId);
        try {
            Element root = doc.getDocumentElement();
            
//          Information about Notification
            Element notificationAddress = doc.createElementNS(NS_NS1, "NotificationAddress");

            if (senderProb.getProperty("webserviceUrl") != null) {
                Element webserviceUrl = doc.createElementNS(NS_NS1, "WebserviceURL");
                webserviceUrl.appendChild(doc.createTextNode(senderProb.getProperty("webserviceUrl")));
                notificationAddress.appendChild(webserviceUrl);
            } else if (senderProb.getProperty("email") != null) {
                Element email = doc.createElementNS(NS_NS1, "Email");
                Element eaddress = doc.createElementNS(NS_P, "p:Address");
                eaddress.appendChild(doc.createTextNode(senderProb.getProperty("email")));
                email.appendChild(eaddress);
                notificationAddress.appendChild(email);
            }
            root.appendChild(notificationAddress);

            Element sender = doc.createElementNS(NS_NS1, "Sender");


            if (privateVersender)
            {
            	   Element corperateBody = doc.createElementNS(NS_P, "CorporateBody");
                   Element fullName = doc.createElementNS(NS_P, "p:FullName");
                   fullName.appendChild(doc.createTextNode(senderProb.getProperty("fullName")));
                   corperateBody.appendChild(fullName);
                   sender.appendChild(corperateBody);
                   
            } else {
            	   Element organisation = doc.createElementNS(NS_NS1, "Organisation");
                   Element fullName = doc.createElementNS(NS_P, "p:FullName");
                   fullName.appendChild(doc.createTextNode(senderProb.getProperty("fullName")));
                   organisation.appendChild(fullName);
            	   sender.appendChild(organisation);
            	   
            } 
            
            
           
            Element postalAddress = doc.createElementNS(NS_P, "p:Address");
            postalAddress.setAttributeNS(NS_XSI, "xsi:type", "p:PostalAddressType");
            Element countryCode = doc.createElementNS(NS_P, "p:CountryCode");
            countryCode.appendChild(doc.createTextNode(senderProb.getProperty("countryCode")));
            Element postalCode = doc.createElementNS(NS_P, "p:PostalCode");
            postalCode.appendChild(doc.createTextNode(senderProb.getProperty("postalCode")));
            Element municipality = doc.createElementNS(NS_P, "p:Municipality");
            municipality.appendChild(doc.createTextNode(senderProb.getProperty("municipality")));
            
            Element deliveryAddress = doc.createElementNS(NS_P, "p:DeliveryAddress");
            Element streetName = doc.createElementNS(NS_P, "p:StreetName");
            streetName.appendChild(doc.createTextNode(senderProb.getProperty("streetName")));
            Element buildingNumber = doc.createElementNS(NS_P, "p:BuildingNumber");
            buildingNumber.appendChild(doc.createTextNode(senderProb.getProperty("buildingNumber")));
            Element unit = doc.createElementNS(NS_P, "p:Unit");
            unit.appendChild(doc.createTextNode(senderProb.getProperty("unit")));
            
            deliveryAddress.appendChild(streetName);
            deliveryAddress.appendChild(buildingNumber);
            deliveryAddress.appendChild(unit);
            
            postalAddress.appendChild(countryCode);
            postalAddress.appendChild(postalCode);
            postalAddress.appendChild(municipality);
            postalAddress.appendChild(deliveryAddress);

            Element internetAddress = doc.createElementNS(NS_P, "p:Address");
            internetAddress.setAttributeNS(NS_XSI, "xsi:type", "p:InternetAddressType");
            Element addr = doc.createElementNS(NS_P, "p:Address");
            internetAddress.appendChild(addr);
            addr.appendChild( doc.createTextNode( senderProb.getProperty("senderEmail")));

            
            sender.appendChild(postalAddress);
            sender.appendChild(internetAddress);

            root.appendChild(sender);

            

        } catch (DOMException de) {
            logger.warn("DOMExeption beim erstellen der Senderinformationen", de);
        } catch (Exception e) {
            logger.warn("Exception beim erstellen der Senderinformationen", e);
        }
        return doc;
    }

}
