/*
 * Copyright 2004 Bundeskanzleramt sterreich
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package moazs.external;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Iterator;
import java.util.Properties;
import java.util.Set;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.activation.DataHandler;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Session;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;

import moazs.Constants;
import moazs.api.MoaException;
import moazs.modell.Payload;
import moazs.modell.XmlDocument;
import moazs.util.Base64;
import moazs.util.ByteArrayDataSource;
import moazs.util.MiscUtil;
import moazs.util.MoaConfig;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.log4j.Logger;
import org.bouncycastle.mail.smime.SMIMEEnvelopedGenerator;

/**
* Klasse zum verschlsseln von Zustellstcken im SMIME oder Zuse format
*/

public class MoaSMIME {
    static private Logger logger = Logger.getLogger(MoaSMIME.class);
    private  String mailbody   = null;
    private MimeMessage result = null; 
    private boolean zusecontainer;
    
    /**
    * Standard Konstruktor
    */
    public MoaSMIME() {
        MoaConfig mcfg = new MoaConfig();
        zusecontainer = mcfg.getZuseContainer();
    }

    /**
    * Funktion zum setzten des Mailbody der in die Nachricht integriert 
    * wird.
    * @param url 
    *        adresse von der der mailbody abgeholt werden kann
    */
    public void setMailBody(String url) {
        try{
            HttpClient client = new HttpClient();
            GetMethod gm = new GetMethod(url);
            client.executeMethod(gm);
            this.mailbody = new String( gm.getResponseBody());
        } catch (java.io.IOException ioe) {
            logger.warn( "IOException", ioe);
        }

    }

    /** 
    * Funktion zum auslesen des Mailbody. Falls zuvor kein mailbody gesetzt worden ist, 
    * wird der in der konfiguration angegebene mailbody aus dem Filesystem geladen.
     * @return String default mailbody
     */
    public String getMailBody() {
        if (this.mailbody == null) {
            try {
                MoaConfig mcfg = new MoaConfig();
                String bodyname = mcfg.getProperty("defaultMailBody");
                if (bodyname == null) {
                    this.mailbody = "";
                    return this.mailbody;
                }
                /*BufferedReader in = new BufferedReader(new FileReader(bodyname));
                StringBuffer buffer = new StringBuffer();
                String dummy = in.readLine();
                while (dummy != null) {
                    buffer.append(dummy);
                    dummy = in.readLine();
                }
                this.mailbody = buffer.toString();*/
                
                // load mail body
                FileInputStream fis = new FileInputStream(bodyname);
                ByteArrayOutputStream bos = new ByteArrayOutputStream();
                MiscUtil.dumpInputOutputStream(fis, bos);
                this.mailbody = bos.toString();
            } catch (java.io.FileNotFoundException fnf) {
                logger.warn( "FileNotFoundException", fnf);
                this.mailbody = "";
            } catch (java.io.IOException ioe) {
                logger.warn( "IOException", ioe);
                this.mailbody = "";
            }

        }
        return this.mailbody;
    }

    /**
    * Generiert eine MultipartMime oder einen Zusecontainer aus den 
    * bergebenen Payloads.
     * @param xml  XmlDocument
     * @param payload Set of Payloads
     * @throws MoaException
     */
    public void makeMessage(XmlDocument xml, Set payload) throws MoaException {
        if (zusecontainer) {
            this.result = makeZuse(xml, payload);
        } else {
            this.result = makeMime(xml, payload);
        }
    }

    /**
    * Verschlsselt die Nachricht in result mit dem bergebenen Zertifikat
     * @param cert bytearray des Zertifikates
     * @throws MoaException
     */
    public void encrypt( byte[] cert) throws MoaException {
        // Key im DER Format einlesen
        X509Certificate cert509 = null;
        boolean error = false;
        try {
            CertificateFactory factory = CertificateFactory.getInstance("X509");
            cert509 = (X509Certificate) factory.generateCertificate( new ByteArrayInputStream( Base64.decode(new String(cert))));
            logger.debug("Encrypting with certificate: " + cert509.getIssuerDN() + "/" + cert509.getSerialNumber());
        } catch (java.security.cert.CertificateException ce) {
            logger.warn( "CertificateException ...", ce);
            throw new MoaException("Zertifikat nicht lesbar");
        } catch (Exception e) {
            logger.warn( "Generali Exception ...", e);
            throw new MoaException("Zertifikat nicht lesbar ...");
            
        }

        try {
            SMIMEEnvelopedGenerator  gen = new SMIMEEnvelopedGenerator();
            gen.addKeyTransRecipient(cert509);
      
            MimeBodyPart mp = null;
            mp = gen.generate(this.result, SMIMEEnvelopedGenerator.DES_EDE3_CBC, "BC");
            Properties props = System.getProperties();
            Session session = Session.getDefaultInstance(props, null);
       
            // smime basteln
            MimeMessage body = new MimeMessage(session);
            body.setContent(mp.getContent(), "application/pkcs7-mime");
            body.saveChanges();

            this.result = body;
        } catch (javax.mail.MessagingException me) {
            logger.warn( "MessagingException ...", me);
            error = true;
        } catch( java.security.NoSuchAlgorithmException nsae) {
            logger.warn( "NoSuchAlgorithmException ...", nsae);
            error = true;
        } catch (java.security.NoSuchProviderException nsp) {
            logger.warn( "NoSuchProviderException ...", nsp);
            error = true;
        } catch (org.bouncycastle.mail.smime.SMIMEException sme) {
            logger.warn( "SMIMEException ...", sme);
            error = true;
        } catch (java.io.IOException ioe) {
            logger.warn( "IOException ...", ioe);
            error = true;
        }
        if (error)  {
            throw new MoaException("Fehler beim Verschlsseln");
        }
    }
   
    /**
    * Gibt die Nachricht als Bytearray aus
     * @return bytearray result
     * @throws MessagingException 
     * @throws IOException 
     */
    public byte[] getResult() throws MessagingException, IOException {
        byte[] ret = null; 
        if (this.result == null) {
            return ret;
        }
        
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        this.result.removeHeader("Message-ID");
        this.result.writeTo( bos);
        ret = bos.toByteArray(); 
        return ret;
    }

    /**
    * funktion zum generieren von Multipart-MIME nachrichten
     * @param xml
     * @param payload
     * @return
     * @throws MoaException
     */
    private MimeMessage makeMime(XmlDocument xml, Set payload) throws MoaException {
        logger.debug("MoaSMIME: MakeMime");
        try {
            Properties props = System.getProperties();
            Session session = Session.getDefaultInstance(props, null);

            Multipart mup = new MimeMultipart();
            
            // Standard body
            MimeBodyPart standardtext = new MimeBodyPart();
            standardtext.setText(getMailBody());
            standardtext.setDisposition("inline");

            mup.addBodyPart(standardtext);

            logger.debug("MoaSMIME: (MakeMime) add Payload");

            if (xml!=null) {
                MimeBodyPart msg = new MimeBodyPart();
                String name = xml.getFilename();
                String mime = xml.getMimetype();
                if (xml.getData() == null) {
                    throw new MoaException("NO PAYLOAD!!!!");
                } 

                logger.debug("XMLCONTENT:" + new String(xml.getDataAsByteArray()));
                msg.setDisposition("attachment; filename=\""+((name==null)?"zustellung.xml":name)+"\"");
                msg.setDataHandler(
                    new DataHandler( 
                        new ByteArrayDataSource( xml.getDataAsByteArray(), (mime==null) ? "text/xml" : mime, (name==null)?"zustellung.xml":name)));
                mup.addBodyPart(msg);


                if (xml.getPstylesheet() !=null) {
                    MimeBodyPart msgps = new MimeBodyPart();
                    String nameps = xml.getPstylesheetName();
                    msgps.setDisposition("attachment; filename=\""+((nameps==null)?"zustellung.xsl":nameps)+"\"");


                    String mimeps = "text/xml";

                    if (xml.getPstylesheet() == null) {
                        throw new MoaException("NO PAYLOAD!!!!");
                    }
                    msgps.setDataHandler(
                        new DataHandler( 
                            new ByteArrayDataSource( xml.getPStyleSheetAsByteArray(), mimeps, (nameps==null)?"zustellung.xsl":nameps)));

                    mup.addBodyPart(msgps);
                }

            }

            if (payload != null && payload.size()!=0) {
                // Mailparts generieren
                for (Iterator i = payload.iterator(); i.hasNext(); ) {
                    Payload p = (Payload)i.next();
                    MimeBodyPart msg = new MimeBodyPart();
                    String name = p.getFilename();
                    String mime = p.getMimetype();
                    
                    if (p.getData() == null) {
                        throw new MoaException("NO PAYLOAD!!!!");
                    }
                    byte[] data = null;
                    
                    if (p.getBase64()==true) {
                        // data = Base64.decode(p.getData());
                        data = Base64.decode( new String(p.getDataAsByteArray()));
                    } else {
                        data = p.getDataAsByteArray();
                    }
                    msg.setDisposition("attachment; filename=\""+((name==null)?"zustellung.xml":name)+"\"");
                    msg.setDataHandler(
                        new DataHandler( 
                            new ByteArrayDataSource( data, (mime==null)?"text/plain":mime, (name==null)?"unnamed":name)));

                    mup.addBodyPart(msg);
                }
            }
            logger.debug("MoaSMIME: (MakeMime) make MimeBody");

            // Nachrict zum verschlsseln basteln
            MimeMessage plainbody = new MimeMessage(session);
            plainbody.setContent(mup);
            plainbody.saveChanges();
             
            return plainbody;
        } catch(javax.mail.MessagingException me) {
          logger.warn( "MessagingException ...", me);
          throw new MoaException(Constants.ERROR_PROCESSING_UNCLASSIFIED, "Unclassified error while handling request.");  
        } catch(IOException e) {
          logger.warn( "IOException", e);
          throw new MoaException(Constants.ERROR_PROCESSING_UNCLASSIFIED, "Unclassified error while handling request.");
        }
    }

    /**
    * Funktion zum generieren von Nachrichten im Zuseformat
     * @param xml
     * @param payload
     * @return
     * @throws MoaException
     */
    private MimeMessage makeZuse(XmlDocument xml, Set payload) throws MoaException {
        logger.debug("MoaSMIME: MakeZuse");

        try{
            Properties props = System.getProperties();
            Session session = Session.getDefaultInstance(props, null);

            // Standard body
            MimeBodyPart standardtext = new MimeBodyPart();
            standardtext.setText(getMailBody());
            standardtext.setDisposition("inline");
            Multipart mup = new MimeMultipart();
            mup.addBodyPart(standardtext);

            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            ZipOutputStream zout = new ZipOutputStream(bos);

            if (xml!=null) {
                String name = xml.getFilename();
                ZipEntry ze = new ZipEntry(name);
                zout.putNextEntry(ze);
                zout.write(xml.getDataAsByteArray());
                zout.closeEntry();     

                if (xml.getPstylesheet() !=null) {
                    String nameps = xml.getPstylesheetName();
                    ZipEntry zeps = new ZipEntry(nameps);
                    zout.putNextEntry(zeps);
                    zout.write(xml.getPStyleSheetAsByteArray());
                    zout.closeEntry();     
                }
            }

            for (Iterator i = payload.iterator(); i.hasNext(); ) {
                Payload p = (Payload)i.next(); 
                String name = p.getFilename();

                if (p.getData() == null) {
                    throw new MoaException("NO PAYLOAD!!!!");
                }
                ZipEntry ze = new ZipEntry((name==null)?"zustellung.bla":name);
                zout.putNextEntry(ze);
                if (p.getBase64()==true) {
                  // zout.write( Base64.decode(p.getData()));
                   zout.write(Base64.decode( new String(p.getDataAsByteArray())));
                } else {
                    zout.write(p.getDataAsByteArray());
                }

                zout.closeEntry();     
            }
            zout.close();

            //InternetHeaders head = new InternetHeaders();
            MimeBodyPart zuse = new MimeBodyPart();
            zuse.setDataHandler(new DataHandler( new ByteArrayDataSource( bos.toByteArray(), "application/x-zip", "foobar")));
            
            mup.addBodyPart(zuse);

            MimeMessage plainbody = new MimeMessage(session);
            plainbody.setContent(mup);
            plainbody.saveChanges();

             return plainbody;

        } catch( java.io.IOException ioe) {
          logger.warn( "IOException", ioe);
          throw new MoaException(Constants.ERROR_PROCESSING_UNCLASSIFIED, "Unclassified error while handling request.");

        } catch (javax.mail.MessagingException me) {
          logger.warn( "MessagingException ...", me);
          throw new MoaException(Constants.ERROR_PROCESSING_UNCLASSIFIED, "Unclassified error while handling request.");
        }

    }
}
