/*
 * Copyright 2004 Bundeskanzleramt sterreich
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package moazs.listener;

import iaik.security.provider.IAIK;

import java.security.Provider;
import java.security.Security;
import java.util.List;

import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import moazs.Constants;
import moazs.api.MoaSync;
import moazs.ssl.IAIKSecureProtocolSocketFactory;
import moazs.util.MoaConfig;
import moazs.util.MoaXMLUtils;
import moazs.watchdog.Watchdog;

import org.apache.commons.httpclient.protocol.Protocol;
import org.apache.log4j.Logger;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

public class MoaListener implements ServletContextListener {
    private static Logger logger = Logger.getLogger(MoaListener.class);
   
    private Watchdog watchdog = null;
    private Thread th = null;

    private String truststore = null;
    private String keystore = null;
    private String truststorepw = null;
    private String truststoretype = null;
    private String keystorepw = null;
    private String keystoretype = null;
    
    private String provider = null;
    private String pkgs = null;

    private BouncyCastleProvider bcProvider = null;
    
    public MoaListener() {
      
      logger.info("Starting MOA-ZS v" + Constants.VERSION);
        
      MoaConfig mcfg = new MoaConfig();

      this.truststore   = mcfg.getProperty("truststore");
      this.truststorepw = mcfg.getProperty("truststorepw") != null ? mcfg.getProperty("truststorepw") : "";
      this.truststoretype = mcfg.getProperty("truststoretype") != null ? mcfg.getProperty("truststoretype") : "JKS";
      this.keystore     = mcfg.getProperty("keystore");
      this.keystorepw   = mcfg.getProperty("keystorepw") != null ? mcfg.getProperty("keystorepw") : "";
      this.keystoretype = mcfg.getProperty("keystoretype") != null ? mcfg.getProperty("keystoretype") : "JKS"; 
      this.provider     = mcfg.getProperty("sslprovider");
      this.pkgs         = mcfg.getProperty("sslpkgs");
    }

    public void initializeSSLContext() {
      try {
        Provider s = (Provider) Class.forName(this.provider).newInstance();
        Security.addProvider(s);
        System.setProperty("java.protocol.handler.pkgs", this.pkgs );
        logger.info("Initializing key- and truststores.");
        if (keystore != null) {
            logger.debug("Keystore: "+keystore);
            logger.debug("KeystorePW: "+keystorepw);
            logger.debug("KeystoreType: "+keystoretype);
            
            System.setProperty("javax.net.ssl.keyStore", keystore);
            System.setProperty("javax.net.ssl.keyStorePassword", keystorepw);
            System.setProperty("javax.net.ssl.keyStoreType", keystoretype);
        }
        if (truststore != null) {
            logger.debug("trustStore: "+truststore);
            System.setProperty("javax.net.ssl.trustStore", truststore);
            System.setProperty("javax.net.ssl.trustStorePassword", truststorepw);
            System.setProperty("javax.net.ssl.trustStoreType", truststoretype);
        }
        
        if ("IAIK".equalsIgnoreCase(truststoretype)) {
          logger.info("Using IAIK as SSL trust provider.");
          
          // settings for AXIS secure communication
          System.setProperty("org.apache.axis.components.net.SecureSocketFactory", "moazs.ssl.IAIKSecureSocketFactory");
          
          // settings for apache httpclient secure communication
          Protocol.registerProtocol("https", new Protocol("https", new IAIKSecureProtocolSocketFactory(), 443));
        } else {
          logger.info("SSL truststoretype: " + truststoretype);
        }
        
      } catch (Exception e) {
          System.out.println("An exception occured during initialization of the security environment");
          e.printStackTrace();
      }      
    }
    
    public void contextInitialized(ServletContextEvent sce) {

      // initialize SSL context
      initializeSSLContext();
      
        watchdog = new Watchdog();
        th = new Thread(watchdog);
        th.start();
        
      // register providers
      logger.debug("Registering security providers.");
      IAIK.addAsJDK14Provider();
      bcProvider = new BouncyCastleProvider();
      Security.addProvider(bcProvider);
      
      // initialize xml utils
      MoaXMLUtils.initialize();
    }

    public void contextDestroyed(ServletContextEvent sce) {
      // MOA-ZS worker
      logger.debug("Shutting down MOA-ZS worker.");
      watchdog.end();
      try {
          th.join();
      } catch ( java.lang.InterruptedException ie) {
        logger.warn("MOA-ZS worker interrupted.", ie);
      }
      
      // shutting down all sync threads
      List syncWatchDogList = MoaSync.getSyncWatchDogThreads();
      logger.debug("Shutting down sync threads (" + syncWatchDogList.size() + " remaining)");
      for (int i=0; i<syncWatchDogList.size(); i++) {
        MoaSync wd = (MoaSync) syncWatchDogList.get(i);
        if (wd.isStopped()) {
          wd.terminate();
          while (!wd.isStopped()) {
            try { Thread.sleep(10); } catch(InterruptedException e) {}
          }
        }
      }
      
      // unregistering security providers
      logger.debug("Unregistering security providers.");
      Security.removeProvider("IAIK");
      Security.removeProvider(bcProvider.getName());
    }
}
