/*
 * Copyright 2004 Bundeskanzleramt sterreich
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package moazs.util;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.logging.FileHandler;
import java.util.logging.SimpleFormatter;
import java.util.logging.XMLFormatter;

import moazs.Constants;
import moazs.api.MoaException;
import moazs.modell.DeliveryQuality;
import moazs.modell.DualDelivery;
import moazs.modell.Identification;
import moazs.modell.Internetaddress;
import moazs.modell.Payload;
import moazs.modell.Postaladdress;
import moazs.modell.QueueInfo;
import moazs.modell.Rcpt;
import moazs.modell.Request;
import moazs.modell.Status;
import moazs.modell.Telephone;
import moazs.modell.XmlDocument;
import moazs.modell.Zuseserver;
import moazs.szr.SZRClient;
import moazs.szr.SZRClientException;

import org.apache.log4j.Logger;
import org.hibernate.HibernateException;
import org.hibernate.Transaction;
import org.hibernate.classic.Session;
import org.jconfig.Category;
import org.jconfig.Configuration;
import org.jconfig.ConfigurationManager;
import org.jconfig.ConfigurationManagerException;
import org.jconfig.handler.XMLFileHandler;
import org.jconfig.parser.NestedConfigParser;

import at.gv.egovernment.moa.util.DOMUtils;


/**
 * Klasse zum einlesen der Konfiguration
 * und Bereitstellung von Parametern
 */
public class MoaConfig {

    private static Configuration conf = null;

    private static final String CONFIG_SYSTEM_PROP = "moazs.configuration";
    
    private static SZRClient szrClient = null;
    
    private static final float PVP_STANDARD_VERSION = 1.8f;
    
    private static Logger logger = Logger.getLogger(MoaConfig.class);
    
    private static List preferredServers = new ArrayList();
    
    private static Hashtable dualDeliveryServers = new Hashtable();
    
    private static List corporateBodies = new ArrayList();
    
    /*
     * Hibernate props
     */
    private static org.hibernate.cfg.Configuration hibernateConf = null;
    private static boolean canOrderSort = false;
    
    /**
     * Constructor
     */
    public MoaConfig() {
      System.setProperty("jconfig.trimValue", "true");
      if (conf == null) {
          try {
            init();
          } catch (MoaException e) {
            e.printStackTrace();
            System.exit(-1);
          }
      }
    }
    
    private void checkMandatoryParameters() throws MoaException {
      // check for sector specific settings
      if (getBereichskennung() == null) {
        throw new MoaException("Parameter target/bereichskennung must not be null. Please check configuration or see sample configuration.");
      }
      if (getBereichskennungEncrypted() == null) {
        throw new MoaException("Parameter target/bereichskennung_ver must not be null. Please check configuration or see sample configuration.");
      }
    }
    
    private void initCorporateBodies() {
      Properties cbp = conf.getProperties("target/corporatebody");
      if (cbp == null || cbp.size() == 0) {
        logger.warn("No corporate bodies defined. Delivery by using Stammzahl for corporate bodies DISABLED!");
        return;
      }
      Enumeration propList = cbp.keys();
      while (propList.hasMoreElements()) {
        String key = (String) propList.nextElement();
        String value = cbp.getProperty(key);
        if (!value.startsWith("urn:publicid:gv.at:baseid+")) {
          logger.warn("Corporate body identifier for '" + key + "' does not start with 'urn:publicid:gv.at:baseid+'");
        } else {
          logger.info("Found corporate body in config: " + value);
          corporateBodies.add(value);
        }
      }
      logger.debug("Total number of corporate bodies found: " + corporateBodies.size());
    }
    
    public String getBereichskennung() {
      return conf.getProperty("bereichskennung", null, "target");
    }
    
    public String getBereichskennungEncrypted() {
      return conf.getProperty("bereichskennung_ver", null, "target");
    }
    
    public List getCorporateBodies() {
      return this.corporateBodies;
    }
    
    public String getGlobalWebServiceUrl() {
      return conf.getProperty("global.webservice.url", null, "general");
    }
    
    // delivery notification (RSa) signature verified via MOA-SP or not
    public boolean verifyNotificationSignature() {
      return conf.getBooleanProperty("verifysignature", true, "deliverynotification");
    }
    
    // for a sender profile a private delivery only property can be set.
    // all request will be interpreted as private only - can be overridden in request
    public boolean isPrivateDeliveryOnly(String senderProfileId) {
      String pdo = (String) getSenderInfo(senderProfileId).get("PrivateDeliveryOnly");
      if (pdo == null) {
        return false;
      }
      return "true".equals(pdo.toLowerCase());
    }

    public boolean isDemoMode() {
      return conf.getBooleanProperty("demo.mode", false);
    }
    
    public String getDeliveryNotificationProperty(String key) {
      return conf.getProperty(key, null, "deliverynotification");
    }
    
    public boolean getDeliveryNotificationBooleanProperty(String key) {
      return conf.getBooleanProperty(key, false, "deliverynotification");
    }
    
    /**
     * Initialisierung der globalen Configuration
     * <p>
     * Configuration kann aus
     * <ul>
     * <li>XML File (default: config.xml)
     * <li>spezifischem XML File (z.B.: moazs_config.xml)
     * <li>Properties File
     * <li>Configuration Server
     * <li>Datenbank (dabei ist allerdings ein jconfig.properties file mit den
     * DB Parametern notwendig)
     * </ul>
     */
    private void init() throws MoaException {
       try { 
        //enable the nested categories
        System.setProperty("jconfig.parser", NestedConfigParser.class.getName());
        
        String systemPropertyConfigFile = System.getProperty(CONFIG_SYSTEM_PROP);
        if (systemPropertyConfigFile != null) {
          logger.debug("Loading configuration via system property " + systemPropertyConfigFile);
          XMLFileHandler handler = new XMLFileHandler();
          handler.setFile(new File(systemPropertyConfigFile));
          conf = handler.load("moazs");
        } else {        
          logger.debug("Searching for configuration in classpath.");
          conf = ConfigurationManager.getConfiguration("moazs");
        }
        
        // if no configuration is found, exit - MOA-ZS cannot be run
        if (conf == null) {
          logger.debug("No configuration found.");
          throw new MoaException("No configuration found.");
        }
        
        // check mandatory parameters
        checkMandatoryParameters();
        
        // Check for demo mode
        if (isDemoMode()) {
          logger.debug("Starting MOA-ZS in demo mode.");
        }
        
        
        // initialize SZR Client
        if (useSZR()) {
          logger.debug("Activating bPK Transformation via Stammzahlenregister.");
          String szrURL = conf.getProperty("endpoint", null, "moaszr");
          if (szrURL == null) {
            throw new MoaException("Endpoint address of Stammzahlenregister not defined.");
          }
          String pvpFilePath = conf.getProperty("pvpHeaderFile", null, "moaszr");
          if (pvpFilePath == null) {
            throw new MoaException("PVP Header file declaration is missing in config.");
          }
          File pvpFile = new File(pvpFilePath);
          if (!pvpFile.exists()) {
            throw new MoaException("PVP Header file does not exist.");
          }
          String pvpVersionString = conf.getProperty("pvpVersion", null, "moaszr");
          float pvpVersion = PVP_STANDARD_VERSION;
          try {
            pvpVersion = Float.parseFloat(pvpVersionString);
          } catch(NumberFormatException e) {}
          
          // check if verfahrenskennzeichen is set
          String vkz = conf.getProperty("vkz", null, "moaszr");
          if (vkz == null) {
            throw new MoaException("No Verfahrenskennzeichen for Stammzahlenregister query defined.");
          }
          
          szrClient = new SZRClient(new URL(szrURL), pvpFilePath, pvpVersion);
          logger.debug("bPK Transformation successfully configured.");
        }

        // adding schemas to pool
        //addSchemasToPool();
        
        // init preferred servers
        initPreferredServers();
        
        // init dual delivery servers
        initDualDeliveryServers();
        
        // init coporate bodies
        initCorporateBodies();
        
        // initialize hibernate
        initHibernate();
        HibernateUtil.initHibernate(this);
        // initialize tables
        initializeTables();
        
       } catch(ConfigurationManagerException e) {
         throw new MoaException(e);
       } catch (MalformedURLException e) {
         throw new MoaException(e);
      } catch (SZRClientException e) {
        throw new MoaException(e);
      }

    }

    public void reload() throws MoaException {
      // just do an init
      init();
    }
    

    

    
    private void initPreferredServers() {
      Category prefCat = conf.getCategory("zustellserver/preferred");
      if (prefCat == null) {
        // nothing to do
        logger.debug("No preferred delivery servers found.");
        return;
      }
      Properties servers = prefCat.getProperties();
      logger.info("Found " + servers.size() + " preferred delivery servers in config.");
      SortedMap smap = new TreeMap();
      Enumeration keys = servers.keys();
      while (keys.hasMoreElements()) {
        String key = (String) keys.nextElement();
        if (key.indexOf(".") == -1) {
          continue;
        }
        int index = -1;
        try {
          index = Integer.parseInt(key.substring(key.indexOf(".") + 1));
        } catch(NumberFormatException e) {
          e.printStackTrace();
          continue;
        }
        smap.put(new Integer(index), servers.get(key));
      }
      
      // create array list
      Iterator it = smap.keySet().iterator();
      while(it.hasNext()) {
        Object serverIndex = it.next();
        logger.info("Adding preferred delivery server (index=" + serverIndex + "): " + smap.get(serverIndex));
        preferredServers.add(smap.get(serverIndex));
      }
    }
    
    private void initDualDeliveryServers() {
      Category dualServerCat = conf.getCategory("dual/servers");
      if (dualServerCat == null) {
        // nothing to do
        logger.debug("No dual delivery servers configured.");
        return;
      }
      Properties servers = dualServerCat.getProperties();
      logger.info("Found " + servers.size() + " dual delivery servers in config.");
      Enumeration keys = servers.keys();
      while (keys.hasMoreElements()) {
        String friendlyName = (String) keys.nextElement();
        String url = servers.getProperty(friendlyName);
        logger.info("Adding dual delivery server: Friendlyname: " + friendlyName + " / URL: " + url);
        dualDeliveryServers.put(friendlyName, url);
      }
    }
    
    public List getPreferredServers() {
      return MoaConfig.preferredServers;
    }
    
    public Hashtable getDualDeliveryServers() {
      return MoaConfig.dualDeliveryServers;
    }
    
    // return URL
    public String getFirstDualDeliveryServer() {
      if (MoaConfig.dualDeliveryServers.size() == 0) {
        return null;
      }
      return (String) MoaConfig.dualDeliveryServers.get(MoaConfig.dualDeliveryServers.keys().nextElement());
    }
    
    public String getDualDeliveryServer(String friendlyName) {
      return (String) MoaConfig.dualDeliveryServers.get(friendlyName);
    }
    
    private void initHibernate() {
      logger.debug("Initializing Hibernate.");
      if (getProperty("hibernate.dialect", "Hibernate").equals("org.hibernate.dialect.FirebirdDialect")) {
          MoaConfig.canOrderSort = true;
      }
      logger.debug("Adding hibernate classes.");
      hibernateConf = new org.hibernate.cfg.Configuration();
      hibernateConf.addClass(QueueInfo.class);
      hibernateConf.addClass(Status.class);
      hibernateConf.addClass(Request.class);
      hibernateConf.addClass(Payload.class);
      hibernateConf.addClass(Rcpt.class);
      hibernateConf.addClass(Identification.class);
      hibernateConf.addClass(Internetaddress.class);
      hibernateConf.addClass(Postaladdress.class);
      hibernateConf.addClass(Telephone.class);
      hibernateConf.addClass(DeliveryQuality.class);
      hibernateConf.addClass(XmlDocument.class);
      hibernateConf.addClass(Zuseserver.class);
      hibernateConf.addClass(DualDelivery.class);
      hibernateConf.addProperties(this.getHibernateProperties());
    }
    
    public org.hibernate.cfg.Configuration getHibernateConfiguration() {
      return hibernateConf;
    }
    
    public boolean canOrderSort() {
      return MoaConfig.canOrderSort;
    }
    
    public String getVKZ() {
      return conf.getProperty("vkz", null, "moaszr");
    }
    
    /*
     * Initializes the tables
     */
    private void initializeTables() {
      logger.debug("Initializing standard tables.");
      Session sess = HibernateUtil.getCurrentSession();
      Transaction tx = sess.beginTransaction();
      
      try {
        logger.debug("Initializing DeliveryQuality table.");
        // delivery quality table
        initialize_deliveryquality();
        
        logger.debug("Initializing status table.");
        // status table
        initialize_status();
        tx.commit();
      } catch(Exception e) {
        logger.warn("Error while initializing tables.", e);
        try {
          if (HibernateUtil.getCurrentSession().getTransaction().isActive()) {
            HibernateUtil.getCurrentSession().getTransaction().rollback();
          }
        } catch(HibernateException err) {
          System.err.println("Could not rollback transaction.");
        }
      }
      
    }

    private void initialize_status() {
      Session s = HibernateUtil.getCurrentSession();
      // first initialize the status table
      Status incoming = HibernateSearch.getStatus(Status.INCOMING);
      if (incoming == null) {
        // DeliveryQuality
        Status st = new Status();
        st.setId(0);
        st.setName(Status.INCOMING);
        s.save(st);
      }
      Status signed = HibernateSearch.getStatus(Status.SIGNED);
      if (signed == null) {
        // DeliveryQuality
        Status st = new Status();
        st.setId(1);
        st.setName(Status.SIGNED);
        s.save(st);
      }
      Status encrypted = HibernateSearch.getStatus(Status.ENCRYPTED);
      if (encrypted == null) {
        // DeliveryQuality
        Status st = new Status();
        st.setId(2);
        st.setName(Status.ENCRYPTED);
        s.save(st);
      }
      Status sent = HibernateSearch.getStatus(Status.SENT);
      if (sent == null) {
        // DeliveryQuality
        Status st = new Status();
        st.setId(3);
        st.setName(Status.SENT);
        s.save(st);
      }
      Status done = HibernateSearch.getStatus(Status.DONE);
      if (done == null) {
        // DeliveryQuality
        Status st = new Status();
        st.setId(4);
        st.setName(Status.DONE);
        s.save(st);
      }
    }
    
    private void initialize_deliveryquality() {
      
      Session s = HibernateUtil.getCurrentSession();
      
      // first initialize the DeliveryQuality table
      DeliveryQuality rsa = HibernateSearch.getDeliveryQuality(DeliveryQuality.RSA);
      if (rsa == null) {
        DeliveryQuality dq = new DeliveryQuality();
        dq.setId(0);
        dq.setName(DeliveryQuality.RSA);
        s.save(dq);
      }

      DeliveryQuality nonRsa = HibernateSearch.getDeliveryQuality(DeliveryQuality.nonRSA);
      if (nonRsa == null) {
        // DeliveryQuality
        DeliveryQuality dq = new DeliveryQuality();
        dq.setId(1);
        dq.setName(DeliveryQuality.nonRSA);
        s.save(dq);
      }

      
      DeliveryQuality rsaPlus = HibernateSearch.getDeliveryQuality(DeliveryQuality.RSA_PLUS);
      if (rsaPlus == null) {
        DeliveryQuality dq = new DeliveryQuality();
        dq.setId(2);
        dq.setName(DeliveryQuality.RSA_PLUS);
        s.save(dq);
      }

      
      DeliveryQuality nonRsaPlus = HibernateSearch.getDeliveryQuality(DeliveryQuality.nonRSA_PLUS);
      if (nonRsaPlus == null) {
        // DeliveryQuality
        DeliveryQuality dq = new DeliveryQuality();
        dq.setId(3);
        dq.setName(DeliveryQuality.nonRSA_PLUS);
        s.save(dq);
      }      

      // dual delivery
      DeliveryQuality rsb = HibernateSearch.getDeliveryQuality(DeliveryQuality.RSB);
      if (rsb == null) {
        DeliveryQuality dq = new DeliveryQuality();
        dq.setId(4);
        dq.setName(DeliveryQuality.RSB);
        s.save(dq);
      }
      
      DeliveryQuality rsbPlus = HibernateSearch.getDeliveryQuality(DeliveryQuality.RSB_PLUS);
      if (rsbPlus == null) {
        DeliveryQuality dq = new DeliveryQuality();
        dq.setId(5);
        dq.setName(DeliveryQuality.RSB_PLUS);
        s.save(dq);
      }

      // private delivery
      DeliveryQuality r = HibernateSearch.getDeliveryQuality(DeliveryQuality.R);
      if (r == null) {
        DeliveryQuality dq = new DeliveryQuality();
        dq.setId(6);
        dq.setName(DeliveryQuality.R);
        s.save(dq);
      }
      
      DeliveryQuality rPlus = HibernateSearch.getDeliveryQuality(DeliveryQuality.R_PLUS);
      if (rPlus == null) {
        DeliveryQuality dq = new DeliveryQuality();
        dq.setId(7);
        dq.setName(DeliveryQuality.R_PLUS);
        s.save(dq);
      }

      DeliveryQuality rs = HibernateSearch.getDeliveryQuality(DeliveryQuality.RS);
      if (rs == null) {
        DeliveryQuality dq = new DeliveryQuality();
        dq.setId(8);
        dq.setName(DeliveryQuality.RS);
        s.save(dq);
      }
      
      DeliveryQuality rsPlus = HibernateSearch.getDeliveryQuality(DeliveryQuality.RS_PLUS);
      if (rsPlus == null) {
        DeliveryQuality dq = new DeliveryQuality();
        dq.setId(9);
        dq.setName(DeliveryQuality.RS_PLUS);
        s.save(dq);
      }
      
    }
    
    /*
     * Gibt an, ob der Zugang zum Stammzahlenregister genutzt werden soll.
     */
    public boolean useSZR() {
      return conf.getBooleanProperty("active", false, "moaszr");
    }
    
    public SZRClient getSZRClient() throws MoaException {
      if (!useSZR()) {
        logger.debug("SZR connection not activated. Please activate first in configuration. Aborting.");
        throw new MoaException("SZR connection not actived. Please activate first in configuration.");
      }

      return szrClient;
    }
    
    
    /**
     * Konfiguriert die Einstellungen fr Hibernate
     * <p>
     * Einstellungen werden aus der Kategorie 'Hibernate' des
     * Konfigurationsfiles gelesen und als Properties bergeben.
     * <p>
     *
     * @return HibernateProperties
     */
    public Properties getHibernateProperties() {
        Category cat = conf.getCategory("Hibernate");
        Properties hprob = new Properties();
        hprob = cat.getProperties();
        return hprob;
    }
 
 
    /**
     * Liest die PVP-Header fr das StammzahlRegister
     * <p>
     * Die Header werden aus der Kategorie "moazsr/pvp-header" des
     * Konfigurationsfiles gelesen und als Properties bergeben.
     * <p>
     *
     * @return PVPHeader
     */
    public Properties getPVPHeader() {
        Category cat = conf.getCategory("moaszr/pvp-header");
        Properties hprob = new Properties();
        hprob = cat.getProperties();
        return hprob;
    }
   
    /**
     * Methode um Werte aus dem Konfigurationsfile zu erhalten
     * <p>
     * Zugriff ist dabei nur auf Werte in der Categorie "general"
     * des Konfigurationsfiles mglich
     * <p>
     * Zugriff erfolgt mit:
     * MoaConfig mcfg = new MoaConfig();
     * String einWert = mcfg.getProperty("einProperty");
     * <p>
     * @param key The Property key within the configuration
     * @return Property value as String
     */
    public String getProperty(String key) {
        return conf.getProperty(key);
    }
    
    public boolean getBooleanProperty(String key, boolean defaultValue) {
      return conf.getBooleanProperty(key, defaultValue);
    }
    
    public int getIntProperty(String key, int defaultValue, String category) {
      return conf.getIntProperty(key, defaultValue, category);
    }

    /**
     * Methode um Werte aus dem Konfigurationsfile zu erhalten
     * <p>
     * Zugriff auf verschachtelte Kategorien ist moeglich
     * <p>
     * Zugriff erfolgt mit:
     * MoaConfig mcfg = new MoaConfig();
     * String einWert = mcfg.getProperty("einProperty", "eine/verschachtelte/Kategorie");
     * <p>
     * @param key The Property key within the configuration
     * @param category The category of requested key  
     * @return Property value as String
     */
    public String getProperty(String key, String category) {
        return conf.getProperty(key,null,category);
    }
    
    public String getProperty(String key, String defaultValue, String category) {
      return conf.getProperty(key, defaultValue, category);
  }
    
    /**
     * Konfiguriert die globalen Einstellungen fr Logging
     * <p>
     * Einstellungen werden aus der Kategorie 'Logging' des Konfigurationsfiles
     * gelesen.
     */
    public void setLogging() {
        FileHandler fh = null;
        try {
            fh = new FileHandler(conf.getProperty("logFile", null, "Logging"),
                    conf.getIntProperty("logFileSize",0,"Logging"),
                    conf.getIntProperty("nrOfLogFiles",1,"Logging"));
            if (conf.getBooleanProperty("useXML",true,"Logging")) {
                fh.setFormatter(new XMLFormatter());
            } else {
                fh.setFormatter(new SimpleFormatter());                
            }            
            
        } catch (SecurityException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
//        // parse LogLevel
//        Level logLevel = Level.parse(conf.getProperty("logLevel", "INFO", "Logging"));
//        
//        boolean useParentHandlers = conf.getBooleanProperty("logToConsole" ,true, "Logging");
//        Logger.getLogger("moazs").setUseParentHandlers(useParentHandlers);
//        if (useParentHandlers) {
//          if (Logger.getLogger("moazs").getParent().getHandlers().length > 0) {
//            Logger.getLogger("moazs").getParent().getHandlers()[0].setLevel(logLevel);
//          }
//        }
//        Logger.getLogger("moazs").addHandler(fh);
//        Logger.getLogger("moazs").setLevel( logLevel);
        
    }

    /**
     * @return
     */
    public boolean getZuseContainer() {
        return ("true".equals(conf.getProperty("ZuseContainer")))?true:false;
    }

    /**
     * @return
     */
    public int getQueueId() {
        return Integer.parseInt(conf.getProperty("QueueID"));
    }

    /**
     * Wartezeit des Watchdog Threads
     * @return int WatchdogSleep in ms
     */
    public int getWatchdogSleep() {
        return conf.getIntProperty("WatchdogSleep", 5000);
    }
   
    public boolean getSchemaValidation() {
      return conf.getBooleanProperty("schema.validation", true);
    }
    
    public boolean getDebugRequests() {
      return conf.getBooleanProperty("debug.requests", false);
    }
    
    /**
     * @return int WachtdogLimit in ms
     */
    public int getWatchdogLimit() {
        return conf.getIntProperty("WatchdogLimit", 15000);
    }
   
    /**
     * Liefert die Url fuer den TestAlive im Watchdog
     * @return String getPingUrl
     */
    public String getPingUrl() {
        return conf.getProperty("PingUrl");
    }

    
    /**
     * Uebergabe einer Propertie, die saemtliche Werte der uebergebenen Kategorie
     * enthaelt.
     * @param category Kategorie welche ausgelesen werden soll
     * @return Properties, welche saemtliche Werte der Kategorie enthalten.
     */
    public Properties getProperties(String category) {
        Category cat = conf.getCategory(category);
        Properties prob = new Properties();
        prob = cat.getProperties();
        return prob;
    }
    
    /**
     * Auslesen von Senderinformationen aus dem Config-File
     * @param profilId Die ID der absendenden Applikation
     * @return Properties mit den Senderinformationen
     */
    public Properties getSenderInfo(String profilId) {
        Category cat = conf.getCategory("SenderProfileIDInfo/" + profilId);
        Properties prob = new Properties();
        prob = cat.getProperties();
        return prob;
    }
    

    protected void setProperty(String key, String value, String category) {
        conf.setProperty(key, value, category);
    }

    public boolean getCheckMimetypes() {
      return getBooleanProperty("check.mimetypes", true);
    }

}
